import React, { useState, useEffect } from 'react';
import { Form, Input, Button, Row, Col, Select } from 'antd';
import styles from './addForm.less';
import PicturesWall from '@/components/Upload/index';
import EditeConfigWrapper from './editConfigFileWrapper';

const { Item } = Form;
const EditForm = props => {
  const { submitCallback, nodeType, info, configFiles } = props;
  const [form] = Form.useForm();
  const [otherForm] = Form.useForm();
  console.log(info);
  const layout = {
    layout: 'horizontal',
    labelCol: { span: 4, offset: 1 },
    wrapperCol: { span: 16 },
  };
  // 回显表单
  useEffect(() => {
    form.resetFields();
    otherForm.resetFields();
  }, [info]);
  useEffect(() => {
    const targetForm = nodeType === 1 ? form : otherForm;
    let arr = Object.keys(targetForm.getFieldsValue());
    let obj = {};
    arr.map(i => {
      obj[i] = info[i];
    });
    targetForm.setFieldsValue({ ...obj, shortName: info.menuShortName });
  }, [info]);

  const submit = () => {
    const targetForm = nodeType === 1 ? form : otherForm;
    let obj = targetForm.getFieldsValue();
    submitCallback(obj);
  };

  const onFinish = () => {
    submit();
  };
  return (
    <div style={{ marginTop: '10px' }}>
      {nodeType === 1 && (
        <Form
          form={form}
          name="editGroup"
          {...layout}
          className={styles.formStyle}
          onFinish={onFinish}
        >
          <Item
            label="菜单名称"
            name="menuName"
            rules={[
              {
                required: true,
                message: '请输入菜单名称',
              },
            ]}
          >
            <Input />
          </Item>
          <Item label="菜单别名" name="shortName">
            <Input />
          </Item>
          <Item
            label="菜单图标"
            name="imageUrl"
            rules={[
              {
                required: true,
                message: '请选择在线图标',
              },
            ]}
          >
            <PicturesWall />
          </Item>
          <Item label="功能路径" name="pageUrl">
            <Input />
          </Item>
          <Item label="配置文件" name="config">
            <EditeConfigWrapper>
              <Select
                allowClear
                showSearch
                filterOption={(input, option) =>
                  option.children.toLowerCase().indexOf(input.toLowerCase()) >=
                  0
                }
              >
                {configFiles.map(c => (
                  <Select.Option key={c.text} value={c.value}>
                    {c.text}
                  </Select.Option>
                ))}
              </Select>
            </EditeConfigWrapper>
          </Item>
          <Item wrapperCol={{ offset: 5 }} style={{ marginTop: '40px' }}>
            <Button type="primary" htmlType="submit">
              提交
            </Button>
          </Item>
        </Form>
      )}

      {nodeType === 2 && (
        <Form
          form={otherForm}
          name="editMenu"
          {...layout}
          onFinish={onFinish}
          className={styles.formStyle}
        >
          <Item
            label="菜单名称"
            name="menuName"
            rules={[
              {
                required: true,
                message: '请输入菜单名称',
              },
            ]}
          >
            <Input />
          </Item>
          <Item label="菜单别名" name="shortName">
            <Input />
          </Item>
          <Item
            label="菜单图标"
            name="imageUrl"
            rules={[
              {
                required: true,
                message: '请选择菜单图标',
              },
            ]}
          >
            <PicturesWall />
          </Item>
          <Item wrapperCol={{ offset: 5 }} style={{ marginTop: '40px' }}>
            <Button type="primary" htmlType="submit">
              提交
            </Button>
          </Item>
        </Form>
      )}
    </div>
  );
};
export default EditForm;
